#include "mex.h"

#define MAXVALUE +1.0E20
#define MINVALUE -1.0E20

void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])

{
  const mxArray *module_element_ptr; /* Pointer to the Modules */
  double *hiperboxes, *X, *y, *yAct;               /* Hiperboxes matrix, input matrix, and output vectors */
  const mwSize *dims;	/* Number of Hiperboxes of each Modules */
  mwIndex N;	/* Size of the input patterns */
  mwIndex K;	/* Number of input patterns */
  
  if (nrhs != 2)
    mexErrMsgTxt("The number of input arguments must be two.");
  
  /* Find the dimensions of the input matrix X */
  N = mxGetM(prhs[1]);
  K = mxGetN(prhs[1]);
  
  /* Get the matrix of input data */
  X = mxGetPr(prhs[1]);
     
  /* Get the number of classes (number of cells) */  
  int num_of_classes;
  num_of_classes = mxGetNumberOfElements(prhs[0]);
/*  mexPrintf("There are %d classes! \n", num_of_classes); */
  
  if(nlhs<1) 
    mexErrMsgTxt("At least one output is required.");
  
  /* Create an mxArray for the output data */
  plhs[0] = mxCreateDoubleMatrix(1, K, mxREAL);
  y = mxGetPr(plhs[0]);
  
  plhs[1] = mxCreateDoubleMatrix(num_of_classes, K, mxREAL);
  yAct = mxGetPr(plhs[1]);
 
  int class, i, j, xi;
  double aux, BoxAct, ModuleMax;	/* BoxAct and ModuleMax yield the hiperbox activation and the largest activation */
  
  for (xi=0; xi<K; xi++) {
    y[xi]= -1;
    ModuleMax = MINVALUE;
    
    /* Computing the individual output of each module. */ 
    for (class=0; class<num_of_classes; class++)  {
      module_element_ptr = mxGetCell(prhs[0], class);
      
      yAct[num_of_classes*xi+class] = MINVALUE;
      
      if (module_element_ptr == NULL) {
	mexPrintf("\tEmpty Module.\n");
      } else {
	dims = mxGetDimensions(module_element_ptr);
      
	if (dims[0] != 2*N)
	  mexErrMsgTxt("The dimension of the hiperboxes does not agree with the input patterns.");
          
	/* create a pointer to the hiperboxes of this module */
	hiperboxes = mxGetPr(module_element_ptr);
	for (j=0; j<dims[1]; j++){
	  BoxAct = MAXVALUE;

	  for (i=0; i<N; i++){
	   
	    aux= X[N*xi+i] - hiperboxes[dims[0]*j+i]; /* Erosion */
	    if (aux<BoxAct)
	      BoxAct = aux;
	    
	    aux= hiperboxes[dims[0]*j+i+N]-X[N*xi+i]; /* Anti-dilation */
	    if (aux<BoxAct)
	      BoxAct = aux;
	  }
	  
	  if (yAct[num_of_classes*xi+class]<BoxAct){
	    yAct[num_of_classes*xi+class] = BoxAct;
	    if (ModuleMax<BoxAct) {
	      ModuleMax = BoxAct;
	      y[xi] = class;
	    }
	  }
	}
      }
    }
 }
}
