function [Hipercaixas, Error] = gan(Nboxes, Str, dtr, Seval, deval)
% =============== Function =============== %
%  Inputs
% 	Nboxes = [M1 ... MC]
% 	Nboxes = 1 x Number of Classes (C)
%  
%  	Str = [(x1 x2 ... xK)]
%  	dtr = [1 ... K]
%  	xk = N x 1
%  	N = dimension of the problem
%  	K = number of patterns to train
%  	Str = N x K
%  
%  	Seval = [(x1 x2 ... xKeval)]
%  	deval = [1 ... Keval]
%  	Keval = number of patterns to eval
%  	Seval = N x Keval
%  
%  Outputs
%  	Hipercaixas = 
%  	
%  	Error
% ========================================= %

% =============== Parameters ============== %
% Probability of crossover
pc = 0.3;
    
% Probability of mutation
pm = 0.2;
    
% Number of maximum generations
Gmax = 100;

% Pop_size
Pop_size = 30;
% ========================================= %

% ============ Initialization ============ %
% Dimension of the problem
N = length(unique(dtr));

% Number of patters to train
K = length(dtr);

% Number of evaluation patterns
Keval = length(deval);

% Generation
T = 1;

% Length of the individual
% Total number of neurons
M = sum(Nboxes);

% Error
Error = zeros(3,Gmax);
% ========================================= %

% ============ First Population =========== %
% Run all the population and initializate it
%  
%  Population = 1 x Pop_size
%  Each element is a individual v
%  
%  v is the individual
%  v = 1 x M
%  Each element is a cell that represents
% a hiperbox and the class of it
%  
%  Each individual is initializate as a
%  randomly pattern
Population = cell(1,Pop_size);
for iPop = 1:Pop_size
    v = cell(1,M);
    iM = 1;
    for iNboxes = 1:length(Nboxes)
	for iMc = 1:Nboxes(iNboxes)
	    v{iM} = cell(1,3);
	    dtemp = find(dtr == iNboxes);
	    iRand = randi(length(dtemp));
	    v{iM}{1} = Str(:,dtemp(iRand));
	    v{iM}{2} = v{iM}{1};
	    v{iM}{3} = iNboxes;
	    iM = iM + 1;
	end
    end
    Population{iPop} = v;
end
% ========================================= %

% Run it Gmax times
while T<Gmax
% ================ Fitness ================ %
% Fitness = 1 x Pop_size
%  Calcule of fitness of each individual
% using the MPCL algorithm
Fitness = zeros(1,Pop_size);
for iPop = 1:Pop_size
    errotemp = zeros(1,Keval);
    % It is necessary to put Hipercaixas on
    %  the format of MPCL algorithm
    Hipercaixas = setHipercaixas(Population{iPop},Nboxes);
    for iKeval = 1:Keval	
	% Calculate which pattern is wrong
	errotemp(iKeval) = MPCL(Hipercaixas, Seval(:,iKeval));
    end
    Fitness(iPop) = sum(errotemp+1 ~= deval)/Keval;
end
% ========================================= %

% ================ Error ================== %
Error(:,T) = [min(Fitness); mean(Fitness); max(Fitness)];
%Error(:,T)

% % % ============== Best Answer ============== %
% % [value,Best] = min(Fitness);
% % % It is necessary to put Hipercaixas on
% % %  the format of MPCL algorithm
% % HC_MPCGL = setHipercaixas(Population{Best},Nboxes);
% % % ========================================= %
% % figure(1)
% % clf;
% % hold on;
% % for i = 1:size(HC_MPCGL{1},2)
% %         fill([HC_MPCGL{1}(1,i);HC_MPCGL{1}(1,i);HC_MPCGL{1}(3,i);HC_MPCGL{1}(3,i)],[HC_MPCGL{1}(2,i);HC_MPCGL{1}(4,i);HC_MPCGL{1}(4,i);HC_MPCGL{1}(2,i)],'y')
% % end
% % for i = 1:size(HC_MPCGL{2},2)
% %         fill([HC_MPCGL{2}(1,i);HC_MPCGL{2}(1,i);HC_MPCGL{2}(3,i);HC_MPCGL{2}(3,i)],[HC_MPCGL{2}(2,i);HC_MPCGL{2}(4,i);HC_MPCGL{2}(4,i);HC_MPCGL{2}(2,i)],'g')
% % end
% % plot(Str(1,find(dtr==1)),Str(2,find(dtr==1)),'*b');
% % hold on;
% % plot(Str(1,find(dtr==2)),Str(2,find(dtr==2)),'*r');
% % grid on;
% % title('Hiperboxes MPCGL and Train Data');
% % pause
% % % ========================================= %

% ============ Roulette Wheel ============= %
% Here we will select the new individuals
% 
% Total Fitness of the population
F = sum((1-Fitness),2);

% Probability of a selection for each chromosome
p = (1-Fitness)./F;

%  Cumulative probability for each chromosone
q = cumsum(p);

% New Population
OldPopulation = Population;
for iPop = 1:Pop_size
    % Random float r
    r = rand(1);
    % Search for the q(jPop) such
    % that q(jPop-1) < r < q(jPop)
    for jPop = 1:Pop_size
	if (r <= q(jPop))
	    break
	end
    end
    Population{iPop} = OldPopulation{jPop};
end
% ========================================= %

% ============== Cross Over =============== %
Population = crossover(Population,pc);
% ========================================= %

% =============== Mutation ================ %
%Population = mutation(Population,pm,Str,dtr,Nboxes);
Population = mutation_maxNI(Population,pm,Str,dtr,Nboxes);
% ========================================= %
T = T+1;
end

% ================ Fitness ================ %
% Fitness = 1 x Pop_size
%  Calcule of fitness of each individual
% using the MPCL algorithm
Fitness = zeros(1,Pop_size);
for iPop = 1:Pop_size
    errotemp = zeros(1,Keval);
    % It is necessary to put Hipercaixas on
    %  the format of MPCL algorithm
    Hipercaixas = setHipercaixas(Population{iPop},Nboxes);
    for iKeval = 1:Keval	
	% Calculate which pattern is wrong
	errotemp(iKeval) = MPCL(Hipercaixas, Seval(:,iKeval));
    end
    Fitness(iPop) = sum(errotemp+1 ~= deval)/Keval;
end
% ========================================= %

% ================ Error ================== %
Error(:,T) = [min(Fitness); mean(Fitness); max(Fitness)];
% ========================================= %

% ============== Best Answer ============== %
[value,Best] = min(Fitness);
% It is necessary to put Hipercaixas on
%  the format of MPCL algorithm
Hipercaixas = setHipercaixas(Population{Best},Nboxes);
% ========================================= %